package java.nio.file;

import java.util.ServiceLoader;
import java.security.AccessController;
import sun.nio.fs.DefaultFileTypeDetector;
import java.security.PrivilegedAction;
import java.util.function.BiPredicate;
import java.util.stream.StreamSupport;
import java.util.Spliterators;
import java.util.stream.Stream;
import java.nio.charset.CharsetEncoder;
import java.util.ArrayList;
import java.util.List;
import java.nio.channels.Channels;
import java.util.Arrays;
import java.io.OutputStreamWriter;
import java.io.BufferedWriter;
import java.nio.charset.StandardCharsets;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.nio.charset.Charset;
import java.nio.file.attribute.BasicFileAttributeView;
import java.nio.file.attribute.FileTime;
import java.nio.file.attribute.FileOwnerAttributeView;
import java.nio.file.attribute.UserPrincipal;
import java.nio.file.attribute.PosixFileAttributeView;
import java.nio.file.attribute.PosixFileAttributes;
import java.nio.file.attribute.PosixFilePermission;
import java.util.Map;
import java.nio.file.attribute.BasicFileAttributes;
import java.nio.file.attribute.FileAttributeView;
import java.nio.file.spi.FileTypeDetector;
import java.util.Objects;
import java.util.Iterator;
import java.util.EnumSet;
import java.util.Collections;
import java.util.HashSet;
import java.nio.channels.SeekableByteChannel;
import java.nio.file.attribute.FileAttribute;
import java.util.Set;
import java.io.OutputStream;
import java.io.InputStream;
import java.io.IOException;
import java.io.UncheckedIOException;
import java.io.Closeable;
import java.nio.file.spi.FileSystemProvider;

public final class Files
{
    private static final int BUFFER_SIZE = 8192;
    private static final int MAX_BUFFER_SIZE = 2147483639;
    
    private Files() {
    }
    
    private static FileSystemProvider provider(Path path) {
        return path.getFileSystem().provider();
    }
    
    private static Runnable asUncheckedRunnable(Closeable closeable) {
        return ()/*  */ -> {
            try {
                p0.close();
            }
            catch (IOException cause) {
                throw new UncheckedIOException(cause);
            }
            return;
        };
    }
    
    public static InputStream newInputStream(Path path, OpenOption... options) throws IOException {
        return provider(path).newInputStream(path, options);
    }
    
    public static OutputStream newOutputStream(Path path, OpenOption... options) throws IOException {
        return provider(path).newOutputStream(path, options);
    }
    
    public static SeekableByteChannel newByteChannel(Path path, Set<? extends OpenOption> set, FileAttribute<?>... array) throws IOException {
        return provider(path).newByteChannel(path, set, array);
    }
    
    public static SeekableByteChannel newByteChannel(Path path, OpenOption... elements) throws IOException {
        HashSet c = new HashSet(elements.length);
        Collections.addAll(c, elements);
        return newByteChannel(path, c, new FileAttribute[0]);
    }
    
    public static DirectoryStream<Path> newDirectoryStream(Path path) throws IOException {
        return provider(path).newDirectoryStream(path, Files.AcceptAllFilter.FILTER);
    }
    
    public static DirectoryStream<Path> newDirectoryStream(Path path, String str) throws IOException {
        if (str.equals("*")) {
            return newDirectoryStream(path);
        }
        FileSystem fileSystem = path.getFileSystem();
        return fileSystem.provider().newDirectoryStream(path, new DirectoryStream.Filter<Path>() {
            
            @Override
            public boolean accept(Path path) {
                return matcher.matches(path.getFileName());
            }
        });
    }
    
    public static DirectoryStream<Path> newDirectoryStream(Path path, DirectoryStream.Filter<? super Path> filter) throws IOException {
        return provider(path).newDirectoryStream(path, filter);
    }
    
    public static Path createFile(Path path, FileAttribute<?>... array) throws IOException {
        newByteChannel(path, EnumSet.of(StandardOpenOption.CREATE_NEW, StandardOpenOption.WRITE), array).close();
        return path;
    }
    
    public static Path createDirectory(Path path, FileAttribute<?>... array) throws IOException {
        provider(path).createDirectory(path, array);
        return path;
    }
    
    public static Path createDirectories(Path absolutePath, FileAttribute<?>... array) throws IOException {
        try {
            createAndCheckIsDirectory(absolutePath, array);
            return absolutePath;
        }
        catch (FileAlreadyExistsException ex) {
            throw ex;
        }
        catch (IOException ex2) {
            SecurityException ex3 = null;
            try {
                absolutePath = absolutePath.toAbsolutePath();
            }
            catch (SecurityException ex4) {
                ex3 = ex4;
            }
            Path path = absolutePath.getParent();
            while (path != null) {
                try {
                    provider(path).checkAccess(path, new AccessMode[0]);
                    break;
                }
                catch (NoSuchFileException ex5) {
                    path = path.getParent();
                }
            }
            if (path == null) {
                if (ex3 == null) {
                    throw new FileSystemException(absolutePath.toString(), null, "Unable to determine if root directory exists");
                }
                throw ex3;
            }
            else {
                Iterator<Path> iterator = path.relativize(absolutePath).iterator();
                while (iterator.hasNext()) {
                    path = path.resolve((Path)iterator.next());
                    createAndCheckIsDirectory(path, array);
                }
                return absolutePath;
            }
        }
    }
    
    private static void createAndCheckIsDirectory(Path path, FileAttribute<?>... array) throws IOException {
        try {
            createDirectory(path, array);
        }
        catch (FileAlreadyExistsException ex) {
            if (!isDirectory(path, new LinkOption[] { LinkOption.NOFOLLOW_LINKS })) {
                throw ex;
            }
        }
    }
    
    public static Path createTempFile(Path obj, String s, String s2, FileAttribute<?>... array) throws IOException {
        return TempFileHelper.createTempFile((Path)Objects.requireNonNull(obj), s, s2, array);
    }
    
    public static Path createTempFile(String s, String s2, FileAttribute<?>... array) throws IOException {
        return TempFileHelper.createTempFile(null, s, s2, array);
    }
    
    public static Path createTempDirectory(Path obj, String s, FileAttribute<?>... array) throws IOException {
        return TempFileHelper.createTempDirectory((Path)Objects.requireNonNull(obj), s, array);
    }
    
    public static Path createTempDirectory(String s, FileAttribute<?>... array) throws IOException {
        return TempFileHelper.createTempDirectory(null, s, array);
    }
    
    public static Path createSymbolicLink(Path link, Path target, FileAttribute<?>... attrs) throws IOException {
        provider(link).createSymbolicLink(link, target, attrs);
        return link;
    }
    
    public static Path createLink(Path link, Path existing) throws IOException {
        provider(link).createLink(link, existing);
        return link;
    }
    
    public static void delete(Path path) throws IOException {
        provider(path).delete(path);
    }
    
    public static boolean deleteIfExists(Path path) throws IOException {
        return provider(path).deleteIfExists(path);
    }
    
    public static Path copy(Path path, Path path2, CopyOption... array) throws IOException {
        FileSystemProvider provider = provider(path);
        if (provider(path2) == provider) {
            provider.copy(path, path2, array);
            return path2;
        }
        CopyMoveHelper.copyToForeignTarget(path, path2, array);
        return path2;
    }
    
    public static Path move(Path path, Path path2, CopyOption... array) throws IOException {
        FileSystemProvider provider = provider(path);
        if (provider(path2) == provider) {
            provider.move(path, path2, array);
            return path2;
        }
        CopyMoveHelper.moveToForeignTarget(path, path2, array);
        return path2;
    }
    
    public static Path readSymbolicLink(Path link) throws IOException {
        return provider(link).readSymbolicLink(link);
    }
    
    public static FileStore getFileStore(Path path) throws IOException {
        return provider(path).getFileStore(path);
    }
    
    public static boolean isSameFile(Path path, Path path2) throws IOException {
        return provider(path).isSameFile(path, path2);
    }
    
    public static boolean isHidden(Path path) throws IOException {
        return provider(path).isHidden(path);
    }
    
    public static String probeContentType(Path path) throws IOException {
        Iterator<FileTypeDetector> iterator = Files.FileTypeDetectors.installeDetectors.iterator();
        while (iterator.hasNext()) {
            String probeContentType = ((FileTypeDetector)iterator.next()).probeContentType(path);
            if (probeContentType != null) {
                return probeContentType;
            }
            continue;
        }
        return Files.FileTypeDetectors.defaultFileTypeDetector.probeContentType(path);
    }
    
    public static <V extends FileAttributeView> V getFileAttributeView(Path path, Class<V> clazz, LinkOption... array) {
        return provider(path).getFileAttributeView(path, clazz, array);
    }
    
    public static <A extends BasicFileAttributes> A readAttributes(Path path, Class<A> clazz, LinkOption... array) throws IOException {
        return provider(path).readAttributes(path, clazz, array);
    }
    
    public static Path setAttribute(Path path, String s, Object o, LinkOption... array) throws IOException {
        provider(path).setAttribute(path, s, o, array);
        return path;
    }
    
    public static Object getAttribute(Path path, String s, LinkOption... array) throws IOException {
        if (s.indexOf(42) >= 0 || s.indexOf(44) >= 0) {
            throw new IllegalArgumentException(s);
        }
        Map<String, Object> attributes = readAttributes(path, s, array);
        assert !(attributes.size() == 0);
        int index = s.indexOf(58);
        if (index == -1) {
        }
        else {
            s = ((index == s.length()) ? "" : s.substring(index + 1));
        }
        return attributes.get(s);
    }
    
    public static Map<String, Object> readAttributes(Path path, String s, LinkOption... array) throws IOException {
        return provider(path).readAttributes(path, s, array);
    }
    
    public static Set<PosixFilePermission> getPosixFilePermissions(Path path, LinkOption... array) throws IOException {
        return ((PosixFileAttributes)readAttributes(path, PosixFileAttributes.class, array)).permissions();
    }
    
    public static Path setPosixFilePermissions(Path path, Set<PosixFilePermission> permissions) throws IOException {
        PosixFileAttributeView posixFileAttributeView = (PosixFileAttributeView)getFileAttributeView(path, PosixFileAttributeView.class, new LinkOption[0]);
        if (posixFileAttributeView == null) {
            throw new UnsupportedOperationException();
        }
        posixFileAttributeView.setPermissions(permissions);
        return path;
    }
    
    public static UserPrincipal getOwner(Path path, LinkOption... array) throws IOException {
        FileOwnerAttributeView fileOwnerAttributeView = (FileOwnerAttributeView)getFileAttributeView(path, FileOwnerAttributeView.class, array);
        if (fileOwnerAttributeView == null) {
            throw new UnsupportedOperationException();
        }
        return fileOwnerAttributeView.getOwner();
    }
    
    public static Path setOwner(Path path, UserPrincipal owner) throws IOException {
        FileOwnerAttributeView fileOwnerAttributeView = (FileOwnerAttributeView)getFileAttributeView(path, FileOwnerAttributeView.class, new LinkOption[0]);
        if (fileOwnerAttributeView == null) {
            throw new UnsupportedOperationException();
        }
        fileOwnerAttributeView.setOwner(owner);
        return path;
    }
    
    public static boolean isSymbolicLink(Path path) {
        try {
            return readAttributes(path, BasicFileAttributes.class, new LinkOption[] { LinkOption.NOFOLLOW_LINKS }).isSymbolicLink();
        }
        catch (IOException ex) {
            return false;
        }
    }
    
    public static boolean isDirectory(Path path, LinkOption... array) {
        try {
            return readAttributes(path, BasicFileAttributes.class, array).isDirectory();
        }
        catch (IOException ex) {
            return false;
        }
    }
    
    public static boolean isRegularFile(Path path, LinkOption... array) {
        try {
            return readAttributes(path, BasicFileAttributes.class, array).isRegularFile();
        }
        catch (IOException ex) {
            return false;
        }
    }
    
    public static FileTime getLastModifiedTime(Path path, LinkOption... array) throws IOException {
        return readAttributes(path, BasicFileAttributes.class, array).lastModifiedTime();
    }
    
    public static Path setLastModifiedTime(Path path, FileTime fileTime) throws IOException {
        ((BasicFileAttributeView)getFileAttributeView(path, BasicFileAttributeView.class, new LinkOption[0])).setTimes(fileTime, null, null);
        return path;
    }
    
    public static long size(Path path) throws IOException {
        return readAttributes(path, BasicFileAttributes.class, new LinkOption[0]).size();
    }
    
    private static boolean followLinks(LinkOption... array) {
        boolean n = true;
        int length = array.length;
        int i = 0;
        while (i < length) {
            LinkOption linkOption = array[i];
            if (linkOption == LinkOption.NOFOLLOW_LINKS) {
                n = false;
                ++i;
            }
            else {
                if (linkOption == null) {
                    throw new NullPointerException();
                }
                throw new AssertionError("Should not get here");
            }
        }
        return n;
    }
    
    public static boolean exists(Path path, LinkOption... array) {
        try {
            if (followLinks(array)) {
                provider(path).checkAccess(path, new AccessMode[0]);
            }
            else {
                readAttributes(path, BasicFileAttributes.class, new LinkOption[] { LinkOption.NOFOLLOW_LINKS });
            }
            return true;
        }
        catch (IOException ex) {
            return false;
        }
    }
    
    public static boolean notExists(Path path, LinkOption... array) {
        try {
            if (followLinks(array)) {
                provider(path).checkAccess(path, new AccessMode[0]);
            }
            else {
                readAttributes(path, BasicFileAttributes.class, new LinkOption[] { LinkOption.NOFOLLOW_LINKS });
            }
            return false;
        }
        catch (NoSuchFileException ex) {
            return true;
        }
        catch (IOException ex2) {
            return false;
        }
    }
    
    private static boolean isAccessible(Path path, AccessMode... array) {
        try {
            provider(path).checkAccess(path, array);
            return true;
        }
        catch (IOException ex) {
            return false;
        }
    }
    
    public static boolean isReadable(Path path) {
        return isAccessible(path, new AccessMode[] { AccessMode.READ });
    }
    
    public static boolean isWritable(Path path) {
        return isAccessible(path, new AccessMode[] { AccessMode.WRITE });
    }
    
    public static boolean isExecutable(Path path) {
        return isAccessible(path, new AccessMode[] { AccessMode.EXECUTE });
    }
    
    public static Path walkFileTree(Path path, Set<FileVisitOption> set, int n, FileVisitor<? super Path> fileVisitor) throws IOException {
        try (final FileTreeWalker fileTreeWalker = new FileTreeWalker(set, n)) {
            FileTreeWalker.Event event = fileTreeWalker.walk(path);
            do {
                FileVisitResult obj;
                switch (event.type()) {
                    case ENTRY: {
                        IOException ioeException = event.ioeException();
                        if (ioeException == null) {
                            assert event.attributes() != null;
                            obj = fileVisitor.visitFile(event.file(), event.attributes());
                            break;
                        }
                        else {
                            obj = fileVisitor.visitFileFailed(event.file(), ioeException);
                            break;
                        }
                    }
                    case START_DIRECTORY: {
                        obj = fileVisitor.preVisitDirectory(event.file(), event.attributes());
                        if (obj == FileVisitResult.SKIP_SUBTREE || obj == FileVisitResult.SKIP_SIBLINGS) {
                            fileTreeWalker.pop();
                            break;
                        }
                        break;
                    }
                    case END_DIRECTORY: {
                        obj = fileVisitor.postVisitDirectory(event.file(), event.ioeException());
                        if (obj == FileVisitResult.SKIP_SIBLINGS) {
                            obj = FileVisitResult.CONTINUE;
                            break;
                        }
                        break;
                    }
                    default: {
                        throw new AssertionError("Should not get here");
                    }
                }
                if (Objects.requireNonNull(obj) != FileVisitResult.CONTINUE) {
                    if (obj == FileVisitResult.TERMINATE) {
                        break;
                    }
                    if (obj == FileVisitResult.SKIP_SIBLINGS) {
                        fileTreeWalker.skipRemainingSiblings();
                    }
                }
                event = fileTreeWalker.next();
            } while (event != null);
        }
        return path;
    }
    
    public static Path walkFileTree(Path path, FileVisitor<? super Path> fileVisitor) throws IOException {
        return walkFileTree(path, EnumSet.noneOf(FileVisitOption.class), Integer.MAX_VALUE, fileVisitor);
    }
    
    public static BufferedReader newBufferedReader(Path path, Charset charset) throws IOException {
        return new BufferedReader(new InputStreamReader(newInputStream(path, new OpenOption[0]), charset.newDecoder()));
    }
    
    public static BufferedReader newBufferedReader(Path path) throws IOException {
        return newBufferedReader(path, StandardCharsets.UTF_8);
    }
    
    public static BufferedWriter newBufferedWriter(Path path, Charset charset, OpenOption... array) throws IOException {
        return new BufferedWriter(new OutputStreamWriter(newOutputStream(path, array), charset.newEncoder()));
    }
    
    public static BufferedWriter newBufferedWriter(Path path, OpenOption... array) throws IOException {
        return newBufferedWriter(path, StandardCharsets.UTF_8, array);
    }
    
    private static long copy(InputStream inputStream, OutputStream outputStream) throws IOException {
        long n = 0L;
        int read;
        for (byte[] array = new byte[8192]; (read = inputStream.read(array)) > 0; n += (long)read) {
            outputStream.write(array, 0, read);
        }
        return n;
    }
    
    public static long copy(InputStream obj, Path path, CopyOption... array) throws IOException {
        Objects.requireNonNull(obj);
        boolean n = false;
        int length = array.length;
        int i = 0;
        while (i < length) {
            CopyOption obj2 = array[i];
            if (obj2 == StandardCopyOption.REPLACE_EXISTING) {
                n = true;
                ++i;
            }
            else {
                if (obj2 == null) {
                    throw new NullPointerException("options contains 'null'");
                }
                throw new UnsupportedOperationException(obj2 + " not supported");
            }
        }
        SecurityException ex = null;
        if (!(n == 0)) {
            try {
                deleteIfExists(path);
            }
            catch (SecurityException ex2) {
                ex = ex2;
            }
        }
        OutputStream outputStream;
        try {
            outputStream = newOutputStream(path, new OpenOption[] { StandardOpenOption.CREATE_NEW, StandardOpenOption.WRITE });
        }
        catch (FileAlreadyExistsException ex3) {
            if (ex != null) {
                throw ex;
            }
            throw ex3;
        }
        Throwable t = null;
        try {
            long copy = copy(obj, outputStream);
            if (outputStream != null) {
                if (t != null) {
                    try {
                        outputStream.close();
                    }
                    catch (Throwable exception) {
                        t.addSuppressed(exception);
                    }
                }
                else {
                    outputStream.close();
                }
            }
            return copy;
        }
        catch (Throwable t2) {
            t = t2;
            throw t2;
        }
        finally {
            if (outputStream != null) {
                if (t != null) {
                    try {
                        outputStream.close();
                    }
                    catch (Throwable exception2) {
                        t.addSuppressed(exception2);
                    }
                }
                else {
                    outputStream.close();
                }
            }
        }
    }
    
    public static long copy(Path path, OutputStream obj) throws IOException {
        Objects.requireNonNull(obj);
        try (final InputStream inputStream = newInputStream(path, new OpenOption[0])) {
            return copy(inputStream, obj);
        }
    }
    
    private static byte[] read(InputStream inputStream, int max) throws IOException {
        byte[] copy = new byte[max];
        int n = 0;
        while (true) {
            int read;
            if ((read = inputStream.read(copy, n, max - n)) > 0) {
                n += read;
            }
            else {
                int read2;
                if (read >= 0 && (read2 = inputStream.read()) >= 0) {
                    if (max <= 2147483639 - max) {
                        max = Math.max(max << 1, 8192);
                    }
                    else {
                        if (max == 2147483639) {
                            throw new OutOfMemoryError("Required array size too large");
                        }
                        max = 2147483639;
                    }
                    copy = Arrays.copyOf(copy, max);
                    copy[n++] = (byte)read2;
                }
                else {
                    return (max == n) ? copy : Arrays.copyOf(copy, n);
                }
            }
        }
    }
    
    public static byte[] readAllBytes(Path path) throws IOException {
        try (final SeekableByteChannel byteChannel = newByteChannel(path, new OpenOption[0]);
             final InputStream inputStream = Channels.newInputStream(byteChannel)) {
            long size = byteChannel.size();
            if (size > 2147483639L) {
                throw new OutOfMemoryError("Required array size too large");
            }
            return read(inputStream, (int)size);
        }
    }
    
    public static List<String> readAllLines(Path path, Charset charset) throws IOException {
        try (final BufferedReader bufferedReader = newBufferedReader(path, charset)) {
            ArrayList<String> list = new ArrayList();
            while (true) {
                String line = bufferedReader.readLine();
                if (line == null) {
                    break;
                }
                list.add(line);
            }
            return list;
        }
    }
    
    public static List<String> readAllLines(Path path) throws IOException {
        return readAllLines(path, StandardCharsets.UTF_8);
    }
    
    public static Path write(Path path, byte[] array, OpenOption... array2) throws IOException {
        Objects.requireNonNull(array);
        try (final OutputStream outputStream = newOutputStream(path, array2)) {
            int min;
            for (int i = array.length; i > 0; i -= min) {
                min = Math.min(i, 8192);
                outputStream.write(array, i - i, min);
            }
        }
        return path;
    }
    
    public static Path write(Path path, Iterable<? extends CharSequence> obj, Charset charset, OpenOption... array) throws IOException {
        Objects.requireNonNull(obj);
        CharsetEncoder encoder = charset.newEncoder();
        try (final OutputStream outputStream = newOutputStream(path, array);
             final BufferedWriter bufferedWriter = new BufferedWriter(new OutputStreamWriter(outputStream, encoder))) {
            Iterator<Object> iterator = obj.iterator();
            while (iterator.hasNext()) {
                bufferedWriter.append((CharSequence)iterator.next());
                bufferedWriter.newLine();
            }
        }
        return path;
    }
    
    public static Path write(Path path, Iterable<? extends CharSequence> iterable, OpenOption... array) throws IOException {
        return write(path, iterable, StandardCharsets.UTF_8, array);
    }
    
    public static Stream<Path> list(Path path) throws IOException {
        DirectoryStream<Path> directoryStream = newDirectoryStream(path);
        try {
            return (Stream)StreamSupport.stream(Spliterators.spliteratorUnknownSize(new Iterator<Path>() {
                
                @Override
                public boolean hasNext() {
                    try {
                        return delegate.hasNext();
                    }
                    catch (DirectoryIteratorException ex) {
                        throw new UncheckedIOException(ex.getCause());
                    }
                }
                
                @Override
                public Path next() {
                    try {
                        return (Path)delegate.next();
                    }
                    catch (DirectoryIteratorException ex) {
                        throw new UncheckedIOException(ex.getCause());
                    }
                }
            }, 1), false).onClose(asUncheckedRunnable(directoryStream));
        }
        catch (Error | RuntimeException ex) {
            try {
                directoryStream.close();
            }
            catch (IOException exception) {
                try {
                    ex.addSuppressed(exception);
                }
                catch (Throwable t) {}
            }
            throw ex;
        }
    }
    
    public static Stream<Path> walk(Path path, int n, FileVisitOption... array) throws IOException {
        FileTreeIterator iterator = new FileTreeIterator(path, n, array);
        try {
            return ((Stream)StreamSupport.stream(Spliterators.spliteratorUnknownSize(iterator, 1), false).onClose(iterator::close)).map(event/* java.nio.file.FileTreeWalker.Event, */ -> event.file());
        }
        catch (Error | RuntimeException ex) {
            iterator.close();
            throw ex;
        }
    }
    
    public static Stream<Path> walk(Path path, FileVisitOption... array) throws IOException {
        return walk(path, Integer.MAX_VALUE, array);
    }
    
    public static Stream<Path> find(Path path, int n, BiPredicate<Path, BasicFileAttributes> biPredicate, FileVisitOption... array) throws IOException {
        FileTreeIterator iterator = new FileTreeIterator(path, n, array);
        try {
            return ((Stream)StreamSupport.stream(Spliterators.spliteratorUnknownSize(iterator, 1), false).onClose(iterator::close)).filter(event/* java.nio.file.FileTreeWalker.Event, */ -> p2.test(event.file(), event.attributes())).map(event2/* java.nio.file.FileTreeWalker.Event, */ -> event2.file());
        }
        catch (Error | RuntimeException ex) {
            iterator.close();
            throw ex;
        }
    }
    
    public static Stream<String> lines(Path path, Charset charset) throws IOException {
        BufferedReader bufferedReader = newBufferedReader(path, charset);
        try {
            return (Stream)bufferedReader.lines().onClose(asUncheckedRunnable(bufferedReader));
        }
        catch (Error | RuntimeException ex) {
            try {
                bufferedReader.close();
            }
            catch (IOException exception) {
                try {
                    ex.addSuppressed(exception);
                }
                catch (Throwable t) {}
            }
            throw ex;
        }
    }
    
    public static Stream<String> lines(Path path) throws IOException {
        return lines(path, StandardCharsets.UTF_8);
    }
    
    private static class AcceptAllFilter implements DirectoryStream.Filter<Path>
    {
        static final Files.AcceptAllFilter FILTER;
        
        @Override
        public boolean accept(Path path) {
            return true;
        }
        
        static {
            FILTER = new Files.AcceptAllFilter();
        }
    }
    
    private static class FileTypeDetectors
    {
        static final FileTypeDetector defaultFileTypeDetector;
        static final List<FileTypeDetector> installeDetectors;
        
        private static FileTypeDetector createDefaultFileTypeDetector() {
            return (FileTypeDetector)AccessController.doPrivileged(new PrivilegedAction<FileTypeDetector>() {
                @Override
                public FileTypeDetector run() {
                    return DefaultFileTypeDetector.create();
                }
            });
        }
        
        private static List<FileTypeDetector> loadInstalledDetectors() {
            return (List)AccessController.doPrivileged(new PrivilegedAction<List<FileTypeDetector>>() {
                @Override
                public List<FileTypeDetector> run() {
                    ArrayList<E> list = new ArrayList();
                    Iterator<FileTypeDetector> iterator = ServiceLoader.load(FileTypeDetector.class, ClassLoader.getSystemClassLoader()).iterator();
                    while (iterator.hasNext()) {
                        list.add((E)iterator.next());
                    }
                    return list;
                }
            });
        }
        
        static {
            defaultFileTypeDetector = createDefaultFileTypeDetector();
            installeDetectors = loadInstalledDetectors();
        }
    }
}
